package Module::Release::UsePerl;
use strict;

use vars qw($VERSION @EXPORT_OK $KEY);

use Carp qw(carp);

$VERSION   = 1.0;
$KEY = 'useperl';

=head1 NAME

Module::Release::UsePerl - make announcements to your use.perl journal

=head1 SYNOPSIS

This module is not implemented
	
=head1 DESCRIPTION

This module is not implemented

=head1 SOURCE AVAILABILITY

This source is part of a SourceForge project which always has the
latest sources in CVS, as well as all of the previous releases.

	http://sourceforge.net/projects/brian-d-foy/
	
If, for some reason, I disappear from the world, one of the other
members of the project can shepherd this module appropriately.

=head1 AUTHORS

brian d foy, C<< <bdfoy@cpan.org> >>

=head1 COPYRIGHT

Copyright (c) 2004 brian d foy.  All rights reserved.

This program is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

$KEY;
